/* eslint-disable no-return-await */
import moment from 'moment'
import Util from '@/assets/libs/util'
import requesModule from '@/assets/libs/requestModule'
// import continuaTransacaoTefOBJ from '@/assets/tef/ContinuaTransacaoTefOBJ'
import MainTef from '@/assets/tef/MainTef'
import CartController from '@/assets/controllers/CartController'
import SaleController from '@/assets/controllers/SaleController'
import PrinterController from '@/assets/controllers/PrinterController'

const EVENT_CONFIGURE_TEF = '/configure'
const EVENT_START_TRANSACTION_TEF = '/start-transaction'
const EVENT_FINISH_TRANSACTION_TEF = '/finish-transaction'

// Restrições do TAA para TEF - Cartão DEBITO
const additionalParamsDebit = '[10;17;18;19;24;27;28;29;31;32;33;40]' // 16
// Restrições do TAA para TEF - Cartão CREDITO
const additionalParamsCredit = '[10;24;27;28;29;31;32;33;36;40;44]'

const PINPAD_ERROR_MESSAGE = 'Não foi possível configurar o PinPad para iniciar a transação. Verifique se o periféricos está rodando'
const DENIED_BY_AUTH = 'Operação negada pelo autorizador'
const PINPAD_ROUTINES_PROBLEM = 'Problema na execução de alguma das rotinas no pinpad'

let paymentMethods = []
paymentMethods = Util.getLocalConfigs('metodosPagamento')

// Armazena o numero do comprovante de pagamento, que será passado para a Odhen Interface salvar a comanda paga
let paymentNumber = ''

let cancelTefData = {
    nsu: null,
    date: null,
    value: null,
    proof: null
}

let digitalWalletSelected = ''

// Traduz o valor de IDTIPORECE, no formato armazenado no DB da Teknisa, nos valores
// esperados pela DLL do TEF
const generateTefFunctionCode = (IDTIPORECE) => {
    if (IDTIPORECE === '1') {
        return 3
    } else if (IDTIPORECE === 'H') {
        return 122
    } else {
        return 2
    }
}

/**
 * @param endSitef: DSENDIPSITEF da tabela CAIXA
 * @param storeId: CDLOJATEF da tabela CAIXA
 * @param terminalId: CDTERTEF da tabela CAIXA
 */
const configureTEF = async (endSitef, storeId, terminalId, CNPJLoja = null) => {
    let dataToSend = {
        endSitef: endSitef,
        storeId: storeId,
        terminalId: terminalId
    }

    if (CNPJLoja) dataToSend.CNPJLoja = CNPJLoja

    return await requesModule.curlRequest(
        EVENT_CONFIGURE_TEF,
        dataToSend
    ).then(r => {
        if (r && r.data) {
            return r.data
        } else {
            return null
        }
    })
}

/**
 * @param startFunction: se é venda a credito, a debito, cancelamento de venda, etc
 * @param value: valor
 * @param text: numero do cupom referente à venda. Nesse momento o TAA ainda nao tem esse numero
 * @param date: data no formato YYYYMMDD
 * @param time: hora no formato HHmmssSS
 * @param operator: operador
 * @param additionalParams: o que deve ser bloqueado na maquininha
 */
const startTransactionTEF = async (startFunction, value, text, date, time, operator, additionalParams) => {
    return await requesModule.curlRequest(
        EVENT_START_TRANSACTION_TEF,
        {
            startFunction: startFunction,
            value: value,
            text: text,
            date: date,
            time: time,
            operator: operator,
            additionalParams: additionalParams
        }
    ).then(r => r.data)
}

/**
 * @param confirm: indica se a transação deve ser confirmada (1) ou estornada (0)
 * @param text: número do Cupom Fiscal correspondente à venda
 * @param date: Data Fiscal no formato AAAAMMDD (a mesma passada quando iniciou a transacao)
 * @param time: Horário Fiscal no formato HHMMSS (a mesma passada quando iniciou a transacao)
 */
const finishTransactionTEF = async (confirm, text, date, time) => {
    return await requesModule.curlRequest(
        EVENT_FINISH_TRANSACTION_TEF,
        {
            confirm: confirm,
            text: text,
            date: date,
            time: time
        }
    ).then(r => r.data)
}

const proceedToConfigureTef = async () => {
    let endSitef = Util.getLocalConfigs('enderecoIPTEF')
    let storeId = Util.getLocalConfigs('codLojaTEF')
    let terminalId = Util.getLocalConfigs('codigoTerminalTEF')
    let CNPJLoja = Util.getLocalConfigs('storeCNPJ')

    let resConfig

    if (CNPJLoja) {
        resConfig = await configureTEF(endSitef, storeId, terminalId, CNPJLoja)
    } else {
        resConfig = await configureTEF(endSitef, storeId, terminalId)
    }

    return resConfig
}

const buildAdditionalParams = (tefFunctionCode) => {
    let additionalParams = ''

    if (tefFunctionCode === 2) { // Venda débito (cartões refeição, alimentação, etc. também entram nesse caso)
        console.log('Aplicando restrições TEF para cartão de débito')
        additionalParams = additionalParamsDebit
    } else if (tefFunctionCode === 3) { // Venda crédito
        console.log('Aplicando restrições TEF para cartão de crédito')
        additionalParams = additionalParamsCredit
    } else if ((tefFunctionCode === 122)) { // Carteiras Digitais
        additionalParams = '{DevolveStringQRCode=1}'
    }

    return additionalParams
}

const handleTefError = (router) => {
    console.log(PINPAD_ERROR_MESSAGE)
    Util.logComunication('Exception ', {}, PINPAD_ERROR_MESSAGE)
    router.push({ name: 'ErrorScreen', params: { errorType: 'tefError' } })
}

const handleDllError = (retornoDllErro, router) => {
    if (retornoDllErro > 0 && retornoDllErro !== 10000) {
        console.log(DENIED_BY_AUTH)
        Util.logComunication('Exception ', {}, DENIED_BY_AUTH)
        return {
            error: true,
            errorType: 'negada pelo autorizador'
        }
    } else if (retornoDllErro === -43) {
        console.log(PINPAD_ROUTINES_PROBLEM)
        Util.logComunication('Exception ', {}, PINPAD_ROUTINES_PROBLEM)
        router.push({ name: 'ErrorScreen', params: { errorType: 'pinPadError' } })
    } else if (retornoDllErro === -2) {
        return {
            error: true,
            errorType: 'cancelada pelo operador'
        }
    } else {
        router.push({ name: 'ErrorScreen', params: { errorType: 'tefError' } })
    }
}

const finalizaComEstornoAutomatico = async (date, hour, routineName) => {
    // 0 = nao confirma transacao e estorna o valor
    await finishTransactionTEF(0, '', date, hour)
    let message = `Transacao interrompida na rotina de ${routineName} com estorno do valor`
    console.log(message)
    Util.logComunication('Exception ', {}, message)
    CartController.clearbandeiraCartao()
}

export default {

    finishTransactionTEF,

    getPaymentMethods: function () {
        return paymentMethods
    },

    navigateToCheckout: function (router) {
        router.push('checkout')
    },

    setPaymentNumber: function (number) {
        paymentNumber = number
    },

    setCancelTefData: function (nsu, date, value) {
        cancelTefData.nsu = nsu
        cancelTefData.date = date
        cancelTefData.value = value
    },

    setCancelTefProof: function (text) {
        cancelTefData.proof = text
    },

    getCancelTefData: function () {
        return cancelTefData
    },

    clearCancelTefData: function () {
        cancelTefData.nsu = null
        cancelTefData.date = null
        cancelTefData.value = null
        cancelTefData.proof = null
    },

    getDigitalWallet: function () {
        return digitalWalletSelected
    },

    setDigitalWallet: function (walletCode) {
        digitalWalletSelected = walletCode
    },

    clearDigitalWallet: function () {
        digitalWalletSelected = ''
    },

    doPayment: async function (paymentMethod, orderCPF, router) {
        if (paymentMethod.IDTIPORECE === '9' || paymentMethod.IDTIPORECE === 'A') {
            return await SaleController.performSale(paymentNumber, paymentMethod, orderCPF, router)
        } else if (paymentMethod.IDTIPORECE === '4') {
            // Se a forma de pagamento escolhida foi dinheiro, chama a performSale imediatamente
            return await SaleController.performSale(paymentNumber, paymentMethod, orderCPF, router)
        } else {
            let date = moment().format('YYYYMMDD')
            let hour = moment().format('HHmmss')

            let resConfig = await proceedToConfigureTef()

            if (resConfig) {
                let tefSuccess = false
                let retornoDllErro = null
                if (resConfig.result === 0) {
                    let tefFunctionCode = generateTefFunctionCode(paymentMethod.IDTIPORECE)
                    let totalValue = CartController.calculateTotalPrice()
                    totalValue = totalValue.replace('R$', '').trim()

                    let additionalParams = buildAdditionalParams(tefFunctionCode)
                    let resStart = await startTransactionTEF(tefFunctionCode, totalValue, '', date, hour, 'operador', additionalParams)
                    let resultJson = JSON.parse(resStart.result)

                    if (resultJson.retornoDll === 10000) {
                        let isDigitalWalletPayment = tefFunctionCode === 122
                        let resContinua = await MainTef.continuaTransacaoTef(totalValue, null, null, null, null, null, null, false, null, isDigitalWalletPayment)
                        if (!resContinua.error && resContinua.finishedTransaction) {
                            // Tudo correu bem com o pagamento, mas a transação TEF só será finalizada dentro da rotina de
                            // venda. Se a venda através da Odhen Interface for concluída com sucesso, a transação TEF será
                            // confirmada. Se a venda falhar, a transação TEF será finalizada sem confirmação (gera
                            // restituição automática do valor pago)
                            tefSuccess = true
                            return await SaleController.performSale(paymentNumber, paymentMethod, orderCPF, router, date, hour)
                        } else if (resContinua.retornoDll) {
                            retornoDllErro = resContinua.retornoDll
                        }
                    }
                }

                await finalizaComEstornoAutomatico(date, hour, 'pagamento')

                if (!tefSuccess) {
                    let result = handleDllError(retornoDllErro, router)
                    if (result && result.error) return result
                } else {
                    CartController.clearCartAndNavigateHome(router)
                }
            } else {
                handleTefError(router)
            }
        }
    },

    doComplementaryPayment: async function (paymentMethod, orderCPF, router, excessValue) {
        let date = moment().format('YYYYMMDD')
        let hour = moment().format('HHmmss')

        let resConfig = await proceedToConfigureTef()

        if (resConfig) {
            let tefSuccess = false
            let retornoDllErro = null
            if (resConfig.result === 0) {
                let tefFunctionCode = generateTefFunctionCode(paymentMethod.IDTIPORECE)
                excessValue = Util.formatMoney(excessValue)
                excessValue = excessValue.replace('R$', '').trim()

                let additionalParams = buildAdditionalParams(tefFunctionCode)
                let resStart = await startTransactionTEF(tefFunctionCode, excessValue, '', date, hour, 'operador', additionalParams)
                let resultJson = JSON.parse(resStart.result)

                if (resultJson.retornoDll === 10000) {
                    let isDigitalWalletPayment = tefFunctionCode === 122
                    let resContinua = await MainTef.continuaTransacaoTef(excessValue, null, null, null, null, null, null, false, null, isDigitalWalletPayment)
                    if (!resContinua.error && resContinua.finishedTransaction) {
                        // Tudo correu bem com o pagamento, mas a transação TEF só será finalizada dentro da rotina de
                        // venda. Se a venda através da Odhen Interface for concluída com sucesso, a transação TEF será
                        // confirmada. Se a venda falhar, a transação TEF será finalizada sem confirmação (gera
                        // restituição automática do valor pago)
                        tefSuccess = true
                        return {
                            error: false,
                            date: date,
                            hour: hour,
                            paymentNumber: paymentNumber
                        }
                    } else if (resContinua.retornoDll) {
                        retornoDllErro = resContinua.retornoDll
                    }
                }
            }

            await finalizaComEstornoAutomatico(date, hour, 'pagamento complementar')

            if (!tefSuccess) {
                let result = handleDllError(retornoDllErro, router)
                if (result && result.error) return result
            }
        } else {
            handleTefError(router)
        }
    },

    doPrincipalPayment: async function (paymentMethod, orderCPF, router) {
        // performSale é chamada diretamente porque apenas pagamentos de crédito pessoal/débito consumidor chegam nesta função
        let emptyPaymentNumber = ''
        let date = ''
        let hour = ''
        let complementedPayment = true
        return await SaleController.performSale(emptyPaymentNumber, paymentMethod, orderCPF, router, date, hour, complementedPayment)
    },

    cancelTef: async function (nsu, tefDate, value, saleType, nfceCode) {
        console.log('Iniciando rotina de cancelamento de TEF')
        this.setCancelTefData(nsu, tefDate, value)

        const cancelTefFunctionCode = 200
        let date = moment().format('YYYYMMDD')
        let hour = moment().format('HHmmss')

        let resConfig = await proceedToConfigureTef()

        if (resConfig) {
            let tefSuccess = false
            let retornoDllErro = null
            if (resConfig.result === 0) {
                let totalValue = ''
                let additionalParams = ''

                let resStart = await startTransactionTEF(cancelTefFunctionCode, totalValue, nfceCode, date, hour, 'operador', additionalParams)
                let resultJson = JSON.parse(resStart.result)

                if (resultJson.retornoDll === 10000) {
                    let resContinua = await MainTef.continuaTransacaoTef(totalValue, null, null, null, null, null, null, true, saleType)
                    if (!resContinua.error && resContinua.finishedTransaction) {
                        tefSuccess = true
                        let cancelSaleResponse = await SaleController.cancelSale(nfceCode)

                        if (cancelSaleResponse && !cancelSaleResponse.error) {
                            // 1 = confirma transacao
                            await finishTransactionTEF(1, '', date, hour)
                            Util.showInfoModal('Cancelamento de TEF finalizado com sucesso')

                            await PrinterController.printCancel(cancelTefData.proof, cancelSaleResponse, nfceCode)

                            this.clearCancelTefData()
                            return true
                        } else if (cancelSaleResponse && cancelSaleResponse.error) {
                            Util.showInfoModal(cancelSaleResponse.message)
                        }

                        this.clearCancelTefData()
                    } else if (resContinua.retornoDll) {
                        retornoDllErro = resContinua.retornoDll
                    }
                }
            }
            // 0 = nao confirma transacao
            await finishTransactionTEF(0, '', date, hour)
            this.clearCancelTefData()
            console.log('Cancelamento interrompido sem sucesso')

            if (!tefSuccess) {
                if (retornoDllErro > 0 && retornoDllErro !== 10000) {
                    console.log(DENIED_BY_AUTH)
                    Util.logComunication('Exception ', {}, DENIED_BY_AUTH)
                    Util.showInfoModal(DENIED_BY_AUTH)
                } else if (retornoDllErro === -43) {
                    Util.showInfoModal(PINPAD_ROUTINES_PROBLEM)
                    console.log(PINPAD_ROUTINES_PROBLEM)
                } else {
                    console.log('Não foi possível realizar o cancelamento do TEF')
                }
            }
        } else {
            Util.showInfoModal(PINPAD_ERROR_MESSAGE)
            console.log(PINPAD_ERROR_MESSAGE)
        }
    }
}
